@@ -201,7 +201,7 @@ def _load_frotz_lib():
         shutil.copyfile(FROTZ_LIB_PATH, frotz_lib_path)
         frotz_lib = cdll.LoadLibrary(frotz_lib_path)
 
-    frotz_lib.setup.argtypes = [c_char_p, c_int]
+    frotz_lib.setup.argtypes = [c_char_p, c_int, c_char_p, c_int]
     frotz_lib.setup.restype = c_char_p
     frotz_lib.shutdown.argtypes = []
     frotz_lib.shutdown.restype = None
@@ -245,6 +245,11 @@ def _load_frotz_lib():
     frotz_lib.getRAM.argtypes = [c_void_p]
     frotz_lib.getRAM.restype = None
 
+    frotz_lib.get_special_ram_size.argtypes = []
+    frotz_lib.get_special_ram_size.restype = int
+    frotz_lib.get_special_ram.argtypes = [c_void_p]
+    frotz_lib.get_special_ram.restype = None
+
     frotz_lib.get_narrative_text.argtypes = []
     frotz_lib.get_narrative_text.restype = c_char_p
     frotz_lib.set_narrative_text.argtypes = [c_char_p]
@@ -358,29 +363,53 @@ class FrotzEnv():
 
     """
     def __init__(self, story_file, seed=None):
-        if not os.path.isfile(story_file):
-            raise FileNotFoundError(story_file)
-
+        self._cache = {}
         self.frotz_lib = _load_frotz_lib()
-        self.is_fully_supported = bool(self.frotz_lib.is_supported(story_file.encode('utf-8')))
-        if not self.is_fully_supported:
-            msg = ("Game '{}' is not fully supported. Score, move, change"
-                   " detection will be disabled.").format(story_file)
-            warnings.warn(msg, UnsupportedGameWarning)
-
-        self.story_file = story_file.encode('utf-8')
-        md5hash = hashlib.md5(open(self.story_file, 'rb').read()).hexdigest()
-        self.bindings = _load_bindings(md5hash)
-        self.act_gen = TemplateActionGenerator(self.bindings) if self.bindings else None
-        self.seed(seed)
-        self.frotz_lib.setup(self.story_file, self._seed)
-        self.player_obj_num = self.frotz_lib.get_self_object_num()
+        self.load(story_file, seed)
 
     def __del__(self):
         if hasattr(self, 'frotz_lib'):
             self.frotz_lib.shutdown()
             dlclose_func(self.frotz_lib._handle)
 
+    def load(self, story_file, seed=None):
+        '''
+        Loads a Z-Machine game.
+
+        :param story_file: Path to a Z-machine rom file (.z3/.z5/.z6/.z8).
+        :param seed: Seed the random number generator used by the emulator.
+                    Default: use walkthrough's seed if it exists,
+                            otherwise use value of -1 which changes with time.
+        :type story_file: path
+        :type seed: int
+        '''
+        self.story_file = story_file.encode('utf-8')
+
+        if story_file not in self._cache:
+            if not os.path.isfile(story_file):
+                raise FileNotFoundError(story_file)
+
+            with open(story_file, "rb") as f:
+                rom = f.read()
+
+            self.is_fully_supported = bool(self.frotz_lib.is_supported(self.story_file))
+            if not self.is_fully_supported:
+                msg = ("Game '{}' is not fully supported. Score, move, change"
+                    " detection will be disabled.").format(story_file)
+                warnings.warn(msg, UnsupportedGameWarning)
+
+            md5hash = hashlib.md5(rom).hexdigest()
+            self.bindings = _load_bindings(md5hash)
+            self.act_gen = TemplateActionGenerator(self.bindings) if self.bindings else None
+
+            self._cache[story_file] = (rom, self.bindings, self.act_gen)
+
+        rom, self.bindings, self.act_gen = self._cache[story_file]
+
+        self.seed(seed)
+        self.frotz_lib.setup(self.story_file, self._seed, rom, len(rom))
+        self.player_obj_num = self.frotz_lib.get_self_object_num()
+
     def seed(self, seed=None):
         '''
         Changes seed used for the emulator's random number generator.
@@ -408,7 +437,8 @@ class FrotzEnv():
 
         '''
         self.close()
-        obs_ini = self.frotz_lib.setup(self.story_file, self._seed).decode('cp1252')
+        rom, _, _ = self._cache[self.story_file.decode()]
+        obs_ini = self.frotz_lib.setup(self.story_file, self._seed, rom, len(rom)).decode('cp1252')
         score = self.frotz_lib.get_score()
         return obs_ini, {'moves':self.get_moves(), 'score':score}
 
@@ -629,9 +659,11 @@ class FrotzEnv():
 
         """
         import hashlib
-        world_str = ', '.join([str(o) for o in self.get_world_objects(clean=True)])
+        world_objs_str = ', '.join([str(o) for o in self.get_world_objects(clean=True)])
+        special_ram_addrs = self._get_special_ram()
+        world_state_str = world_objs_str + str(special_ram_addrs)
         m = hashlib.md5()
-        m.update(world_str.encode('utf-8'))
+        m.update(world_state_str.encode('utf-8'))
         return m.hexdigest()
 
     def get_moves(self):
@@ -784,20 +816,22 @@ class FrotzEnv():
 
         # Optionally extract objs from the global object tree
         if use_object_tree:
-            surrounding = utl.get_subtree(self.get_player_location().child, self.get_world_objects())
-            player_obj = self.get_player_object()
-            if player_obj in surrounding:
-                surrounding.remove(player_obj)
-            world_objs = []
-            for o in surrounding:
-                name = o.name.split()
-                world_objs.append((o.name, 'PROPN', 'OBJTREE'))
-                if len(name) > 1:
-                    # In the case of multi-word names, we assume first words are adjectives.
-                    for w in name[:-1]:
-                        world_objs.append((w, 'ADJ', 'OBJTREE'))
-                    world_objs.append((name[-1], 'NOUN', 'OBJTREE'))
-            objs = objs.union(world_objs)
+            player_loc = self.get_player_location()
+            if player_loc is not None:
+                surrounding = utl.get_subtree(player_loc.child, self.get_world_objects())
+                player_obj = self.get_player_object()
+                if player_obj in surrounding:
+                    surrounding.remove(player_obj)
+                world_objs = []
+                for o in surrounding:
+                    name = o.name.split()
+                    world_objs.append((o.name, 'PROPN', 'OBJTREE'))
+                    if len(name) > 1:
+                        # In the case of multi-word names, we assume first words are adjectives.
+                        for w in name[:-1]:
+                            world_objs.append((w, 'ADJ', 'OBJTREE'))
+                        world_objs.append((name[-1], 'NOUN', 'OBJTREE'))
+                objs = objs.union(world_objs)
 
         # Filter out the objects that aren't in the dictionary
         dict_words = [w.word for w in self.get_dictionary()]
@@ -842,7 +876,7 @@ class FrotzEnv():
 
         """
         if self.game_over() or self.victory() or self._emulator_halted():
-            return []
+            return {}
         diff2acts = {}
         orig_score = self.get_score()
         state = self.get_state()
@@ -877,3 +911,13 @@ class FrotzEnv():
         ram = np.zeros(ram_size, dtype=np.uint8)
         self.frotz_lib.getRAM(as_ctypes(ram))
         return ram
+
+    def _get_special_ram(self):
+        """
+        Returns a numpy array containing the contents of the special ram addresses for the game.
+
+        """
+        ram_size = self.frotz_lib.get_special_ram_size()
+        ram = np.zeros(ram_size, dtype=np.uint8)
+        self.frotz_lib.get_special_ram(as_ctypes(ram))
+        return ram
